import abc
from jaqpotpy.entities.feature import Feature
from jaqpotpy.entities.meta import MetaInfo
from jaqpotpy.entities.entryid import EntryId
from jaqpotpy.entities.dataentry import DataEntry
from jaqpotpy.dto.pretrained_requirements import PretrainedRequirements
from base64 import b64encode
import pickle

class FeatureDirector:

    def __init__(self):
        self._builder = None

    def construct(self, builder):
        meta_info = MetaInfo()
        self._builder = builder
        meta_info.comments = self._builder.get_comments()
        meta_info.descriptions = self._builder.get_descriptions()
        meta_info.titles = self._builder.get_title()
        meta_info.creators = self._builder.get_creator()
        feature = Feature()
        feature.meta = meta_info.__dict__
        feature.units = self._builder.get_units()
        return feature


class FBuilder(metaclass=abc.ABCMeta):

    def __init__(self):
        self.metainfo = MetaInfo()

    @abc.abstractmethod
    def add_comments(self, comment):
        pass

    @abc.abstractmethod
    def add_descriptions(self, description):
        pass

    @abc.abstractmethod
    def add_title(self, title):
        pass

    @abc.abstractmethod
    def add_creator(self, creator):
        pass

    @abc.abstractmethod
    def set_units(self, units):
        pass

    @abc.abstractmethod
    def get_comments(self):
        pass

    @abc.abstractmethod
    def get_descriptions(self):
        pass

    @abc.abstractmethod
    def get_title(self):
        pass

    @abc.abstractmethod
    def get_creator(self):
        pass

    @abc.abstractmethod
    def get_units(self):
        pass


class FeatureBuilder(FBuilder):
    comments = []
    descriptions = []
    titles = []
    creators = []
    units = None

    def add_comments(self, comment):
        self.comments.clear()
        self.comments.append(comment)

    def add_descriptions(self, description):
        self.descriptions.clear()
        self.descriptions.append(description)

    def add_title(self, title):
        self.titles.clear()
        self.titles.append(title)

    def add_creator(self, creator):
        self.creators.clear()
        self.creators.append(creator)

    def set_units(self, units):
        self.units = units

    def get_comments(self):
        return self.comments

    def get_descriptions(self):
        return self.descriptions

    def get_title(self):
        return self.titles

    def get_creator(self):
        return self.creators

    def get_units(self):
        return self.units


class DataEntryDirector:

    def __init__(self):
        self._builder = None

    def construct(self, builder):
        entryId = EntryId()
        self._builder = builder
        entryId.name = self._builder.get_name()
        entryId.ownerUUID = self._builder.get_owneruuid()
        entryId.type = self._builder.get_type()
        entryId.URI = self._builder.get_uri()
        data_entry = DataEntry()
        data_entry.entryId = entryId.__dict__
        data_entry.values = self._builder.get_values()
        return data_entry


class DBuilder(metaclass=abc.ABCMeta):

    def __init__(self):
        self.dataentry = EntryId()

    @abc.abstractmethod
    def set_name(self, name):
        pass

    @abc.abstractmethod
    def set_owneruuid(self, owner_uuid):
        pass

    @abc.abstractmethod
    def set_type(self, type):
        pass

    @abc.abstractmethod
    def set_uri(self, uri):
        pass

    @abc.abstractmethod
    def set_values(self, value):
        pass

    @abc.abstractmethod
    def get_name(self):
        pass

    @abc.abstractmethod
    def get_owneruuid(self):
        pass

    @abc.abstractmethod
    def get_type(self):
        pass

    @abc.abstractmethod
    def get_uri(self):
        pass

    @abc.abstractmethod
    def get_values(self):
        pass


class DataEntryBuilder(DBuilder):

    name = None
    ownerr_uuid = None
    type = None
    uri = None
    value = None

    def set_name(self, name):
        self.name = name

    def set_owneruuid(self, owner_uuid):
        self.ownerr_uuid = owner_uuid

    def set_type(self, type):
        self.type = type

    def set_uri(self, uri):
        self.uri = uri

    def set_values(self, value):
        self.value = value

    def get_name(self):
        return self.name

    def get_owneruuid(self):
        return self.ownerr_uuid

    def get_type(self):
        return self.type

    def get_uri(self):
        return self.uri

    def get_values(self):
        return self.value


class PretrainedNeedsDirector:

    def __init__(self):
        self._builder = None

    def construct(self, builder):
        pretrained_needs = PretrainedRequirements()
        self._builder = builder
        pretrained_needs.rawModel = self._builder.getRawModel()
        pretrained_needs.independentFeatures = self._builder.getIndependentFeatures()
        pretrained_needs.dependentFeatures = self._builder.getDependendFeatures()
        pretrained_needs.predictedFeatures = self._builder.getDependendFeatures()
        pretrained_needs.implementedWith = self._builder.getImplementedWith()
        pretrained_needs.additionalInfo = self._builder.getAdditionalInfo()
        pretrained_needs.algorithm = self._builder.getAlgorithm()
        pretrained_needs.runtime = self._builder.getRuntime()
        pretrained_needs.description = self._builder.getDescription()
        pretrained_needs.title = self._builder.getTitle()
        return pretrained_needs


class PretrainedNeedsBuilder:
    ENCODING = 'utf-8'
    independendFeatures = None
    dependendFeatures = None
    implementedWith = []
    additionalInfo = None
    algorithm = []
    runtime = []
    description = []
    title = []
    rawModel = []

    def setRawModel(self, model):
        self.rawModel.clear()
        p_mod = pickle.dumps(model)
        raw = b64encode(p_mod)
        raw_model = raw.decode(self.ENCODING)
        # self.rawModel = raw_model
        self.rawModel.append(raw_model)

    def setIndependentFeatures(self, ind_f):
        self.independendFeatures = ind_f

    def setDependendFeatures(self, dep_f):
        self.dependendFeatures = dep_f

    def setImplementedWith(self, impWith):
        self.implementedWith.clear()
        self.implementedWith.append(impWith)

    def setAdditionalInfo(self, add_inf):
        self.additionalInfo = add_inf

    def setAlgorithm(self, algo):
        self.algorithm.clear()
        self.algorithm.append(algo)

    """Sets theImplementedIn of this Pretrained model.
        It correspondes to the runtime that is needed for the algorithm to run
       #
       #
       #     :param implementedIn: The runtime of this Model.
       #     """

    def setRuntime(self, runtime):
        self.runtime.clear()
        self.runtime.append(runtime)

    def setDescription(self, descr):
        self.description.clear()
        self.description.append(descr)

    def setTitle(self, title):
        self.title.clear()
        self.title.append(title)

    def getRawModel(self):
        return self.rawModel

    def getIndependentFeatures(self):
        return self.independendFeatures

    def getDependendFeatures(self):
        return self.dependendFeatures

    def getImplementedWith(self):
        return self.implementedWith

    def getAdditionalInfo(self):
        return self.additionalInfo

    def getAlgorithm(self):
        return self.algorithm

    def getRuntime(self):
        return self.runtime

    def getDescription(self):
        return self.description

    def getTitle(self):
        return self.title

# mb = MetaFeatureBuilder()
# mb.add_comments("Comments")
# mb.add_creator("Creator")
# mb.add_descriptions("Descr")
# mb.add_title("Title")
#
# dire = MetaDirector()
# mf = dire.construct(mb)
#
# print(mf.descriptions, mf.titles, mf.comments)



from jaqpotpy.dto.auth_request import AuthRequest
# from tornado.escape import json_decode
import json
from jaqpotpy.entities.algorithm import Algorithm
import json


def decode_auth(request):
    jsonreq = json.dumps(request)
    au_req = AuthRequest(jsonreq['userName'], jsonreq['authToken'])
    return au_req


def decode_algorithms_simple(request):
    algorithms = json.dumps(request)
    return algorithms


def decode_algorithms_to_class(request):
    algorithms = request
    # algorithms = json.dumps(request)
    algos = []
    for algo in algorithms:
        al = Algorithm(algo)
        algos.append(al)
    return algos


def decode_feature(request):
    feature = json.dumps(request)
    return feature


def decode_dataset(request):
    dataset = json.dumps(request)
    return dataset


def decode_model(response):
    model = json.dumps(response)
    return model


# def decode_auth(request):
#     jsonreq = json_decode(request)
#     au_req = AuthRequest(jsonreq['userName'], jsonreq['authToken'])
#     return au_req
#
#
# def decode_algorithms_simple(request):
#     algorithms = json_decode(request)
#     return algorithms
#
#
# def decode_algorithms_to_class(request):
#     algorithms = json_decode(request)
#     algos = []
#     for algo in algorithms:
#         al = Algorithm(algo)
#         algos.append(al)
#     return algos
#
#
# def decode_feature(request):
#     feature = json_decode(request)
#     return feature
#
#
# def decode_dataset(request):
#     dataset = json_decode(request)
#     return dataset
#
#
# def decode_model(response):
#     model = json_decode(response)
#     return model



from .entryid import EntryId


class DataEntry(object):

    def __init__(self, entryId=None, values=None):

        self.entryId = None
        self.values = None

        if entryId is not None:
            self.entryId = entryId
        if values is not None:
            self.values = values

    # @property
    # def entry_id(self):
    #     """Gets the entry_id of this DataEntry.  # noqa: E501
    #
    #
    #     :return: The entry_id of this DataEntry.  # noqa: E501
    #     :rtype: EntryId
    #     """
    #     return self._entry_id
    #
    # @entry_id.setter
    # def entry_id(self, entry_id):
    #     """Sets the entry_id of this DataEntry.
    #
    #
    #     :param entry_id: The entry_id of this DataEntry.  # noqa: E501
    #     :type: EntryId
    #     """
    #
    #     self._entry_id = entry_id
    #
    # @property
    # def values(self):
    #     """Gets the values of this DataEntry.  # noqa: E501
    #
    #
    #     :return: The values of this DataEntry.  # noqa: E501
    #     :rtype: dict(str, object)
    #     """
    #     return self._values
    #
    # @values.setter
    # def values(self, values):
    #     """Sets the values of this DataEntry.
    #
    #
    #     :param values: The values of this DataEntry.  # noqa: E501
    #     :type: dict(str, object)
    #     """
    #
    #     self._values = values



from tornado import gen, httpclient
from tornado.httputil import HTTPHeaders
import getpass
import urllib.parse
from jaqpotpy.mappers import decode
import requests

algos_path = "algorithm"


def get_allgorithms_sync(baseurl, api_key, start=None, max=None):
    uri = baseurl + algos_path
    h = {'Content-Type': 'application/x-www-form-urlencoded',
         'Accept': 'application/json',
         'Authorization': "Bearer " + api_key}
    r = requests.get(uri, headers=h, verify=False)
    return r.json()


def get_allgorithms_classes(base_url, api_key, start, max):
    uri = base_url + algos_path
    h = {'Content-Type': 'application/x-www-form-urlencoded',
         'Accept': 'application/json',
         'Authorization': "Bearer " + api_key}
    r = requests.get(uri, headers=h, verify=False)
    algos = decode.decode_algorithms_to_class(r.json())
    return algos



# def get_allgorithms_sync(baseurl, api_key, start=None, max=None):
#     uri = baseurl + algos_path
#     jclient = httpclient.HTTPClient()
#     h = HTTPHeaders({'Content-Type': 'application/x-www-form-urlencoded'})
#     h.add('Accept', 'application/json')
#     h.add('Authorization', "Bearer " + api_key)
#     uri += '?'
#     if start is not None:
#         uri += "start="+start
#     if max is not None:
#         uri += "max="+max
#     else:
#         uri += "max="+"10"
#     try:
#         response = jclient .fetch(uri, method='GET', headers=h, validate_cert=False)
#         au_req = decode.decode_algorithms_simple(response.body)
#         return au_req
#     except httpclient.HTTPError as e:
#         print("Error: " + str(e))
#     except Exception as e:
#         print("Error: " + str(e))
#     jclient.close()
#
#
# def get_allgorithms_classes(baseurl, api_key, start=None, max=None):
#     uri = baseurl + algos_path
#     jclient = httpclient.HTTPClient()
#     h = HTTPHeaders({'Content-Type': 'application/x-www-form-urlencoded'})
#     h.add('Accept', 'application/json')
#     h.add('Authorization', "Bearer " + api_key)
#     uri += '?'
#     if start is not None:
#         uri += "start="+start
#     if max is not None:
#         uri += "max="+max
#     else:
#         uri += "max="+"10"
#     try:
#         response = jclient.fetch(uri, method='GET', headers=h, validate_cert=False)
#         au_req = decode.decode_algorithms_to_class(response.body)
#         return au_req
#     except httpclient.HTTPError as e:
#         print("Error: " + str(e))
#     except Exception as e:
#         print("Error: " + str(e))
#     jclient.close()



from jaqpotpy.mappers import decode
import requests


dataset_path = "dataset"


def create_dataset_sync(baseurl, api_key, json_dataset):
    uri = baseurl + dataset_path
    h = {'Content-Type': 'application/json',
         'Accept': 'application/json',
         'Authorization': "Bearer " + api_key}
    try:
        r = requests.post(uri, headers=h, data=json_dataset, verify=False)
        return r.json()
    except Exception as e:
        print("Error http: " + str(e))


# def create_dataset_sync(baseurl, api_key, json_dataset):
#     uri = baseurl + dataset_path
#     jclient = httpclient.HTTPClient()
#     h = HTTPHeaders({'Content-Type': 'application/json'})
#     h.add('Accept', 'application/json')
#     h.add('Authorization', "Bearer " + api_key)
#     try:
#         response = jclient.fetch(uri, method='POST', headers=h, body=json_dataset, validate_cert=False)
#         au_req = decode.decode_dataset(response.body)
#         return au_req
#     except httpclient.HTTPError as e:
#         print("Error http: " + str(e))
#     except Exception as e:
#         print("Error generic: " + str(e))
#     jclient.close()



class MetaInfo:
    def __init__(self):
        self.identifiers = None
        self.comments = None
        self.descriptions = None
        self.titles = None
        self.subjects = None
        self.publishers = None
        self.creators = None
        self.contributors = None
        self.audiences = None
        self.rights = None
        self.sameAs = None
        self.seeAlso = None
        self.hasSources = None
        self.doi = None
        self.date = None



import jaqpotpy.api.login as jaqlogin
import jaqpotpy.api.algorithms_api as alapi
import jaqpotpy.api.dataset_api as data_api
import jaqpotpy.api.models_api as models_api
import jaqpotpy.helpers.jwt as jwtok
import jaqpotpy.helpers.helpers as help
import json
import jaqpotpy.api.feature_api as featapi
from jaqpotpy.helpers.serializer import JaqpotSerializer
# from tornado.ioloop import IOLoop
from jaqpotpy.entities.dataset import Dataset
from jaqpotpy.entities.meta import MetaInfo
from jaqpotpy.entities.featureinfo import FeatureInfo
import pandas as pd
import numpy as np
import http.client as http_client
import getpass

ENCODING = 'utf-8'

# import matplotlib
# matplotlib.use('TkAgg')
# import matplotlib.pyplot as plt


class Jaqpot:
    """
    Deploys sklearn models on Jaqpot.

    Extended description of function.

    Parameters
    ----------
    base_url : The url on which Jaqpot services are deployed

    """

    def __init__(self, base_url):
        self.base_url = base_url
        self.api_key = None
        self.user_id = None
        self.http_client = http_client

    def login(self, username, password):
        """
        Logins on Jaqpot.

        Parameters
        ----------
        username : username
        password : password

        """
        try:
            au_req = jaqlogin.authenticate_sync(self.base_url, username, password)
            self.api_key = au_req['authToken']
            self.user_id = jwtok.decode_jwt(self.api_key).get('sub')
        except Exception as e:
            print("Error: " + str(e))

    def set_api_key(self, api_key):
        """
        Set's api key for authentication on Jaqpot.

        Parameters
        ----------
        api_key : api_key can be retireved from the application after logged in

        """
        self.api_key = api_key
        print("api key is set")

    def request_key(self, username, password):
        """
        Logins on Jaqpot.

        Parameters
        ----------
        username : username
        password : password

        """
        try:
            au_req = jaqlogin.authenticate_sync(self.base_url, username, password)
            self.api_key = au_req['authToken']
            self.user_id = jwtok.decode_jwt(self.api_key).get('sub')
        except Exception as e:
            print("Error: " + str(e))

    def request_key_safe(self):
        """
        Logins on Jaqpot by hiding the users password.

        Parameters
        ----------
        username : username
        password : password

        """
        try:
            username = input("Username: ")
            password = getpass.getpass("Password: ")
            au_req = jaqlogin.authenticate_sync(self.base_url, username, password)
            self.api_key = au_req['authToken']
            self.user_id = jwtok.decode_jwt(self.api_key).get('sub')
        except Exception as e:
            print("Error: " + str(e))

    def my_info(self):
        """
        Prints user's info

        """
        print(jwtok.decode_jwt(self.api_key))
        return jwtok.decode_jwt(self.api_key)

    def get_algorithms(self, start=None, max=None):
        self.check_key()
        try:
            algos = alapi.get_allgorithms_sync(self.base_url, self.api_key, start, max)
            return algos
        except Exception as e:
            print("Error:" + str(e))

    def get_algorithms_classes(self, start=None, max=None):
        self.check_key()
        try:
            algos = alapi.get_allgorithms_classes(self.base_url, self.api_key, start, max)
            return algos
        except Exception as e:
            print("Error:" + str(e))

    def upload_dataset(self, df=None, id=None, title=None, description=None):
        if title is None:
            raise Exception("Please submit title of the dataset")
        if description is None:
            raise Exception("Please submit description of the dataset")
        df_titles = list(df)
        for t in df_titles:
            type_to_c = df[t].dtypes
            if type_to_c == 'int64':
                # print(type_to_c)
                df[t] = df[t].astype(float)
        # print(df.dtypes)
        df = df.replace(np.nan, '', regex=True)
        if type(df).__name__ is not 'DataFrame':
            raise Exception("Cannot form a Jaqpot Dataset. Please provide a Dataframe")
        if id is not None:
            df.set_index(id, inplace=True)
            feat_titles = list(df)
            feats = []
            for f_t in feat_titles:
                fe = help.create_feature(f_t, self.user_id)
                jsonmi = json.dumps(fe, cls=JaqpotSerializer)
                # jsonmi = json.dumps(fe.__dict__)
                feats.append(jsonmi)
        else:
            feat_titles = list(df)
            feats = []
            for f_t in feat_titles:
                fe = help.create_feature(f_t, self.user_id)
                # jsonmi = json.dumps(fe.__dict__)
                jsonmi = json.dumps(fe, cls=JaqpotSerializer)
                feats.append(jsonmi)
        feat_map = {}
        featutes = []
        for feat in feats:
            feat_info = FeatureInfo()
            f = featapi.create_feature_sync(self.base_url, self.api_key, feat)
            feat_uri = self.base_url + "feature/" + f["_id"]
            feat_map[f["meta"]["titles"][0]] = feat_uri
            feat_info.uri = feat_uri
            feat_info.name = f["meta"]["titles"][0]
            featutes.append(feat_info.__dict__)
        dataset = Dataset()
        meta = MetaInfo()
        meta.creators = [self.user_id]
        if title is not None:
            meta.titles = [title]
        meta.descriptions = [description]
        dataset.meta = meta.__dict__
        dataset.totalRows = df.shape[0]
        dataset.totalColumns = df.shape[1]
        dataset.existence = "UPLOADED"
        data_entry = help.create_data_entry(df, feat_map, self.user_id)
        dataset.dataEntry = data_entry
        dataset.features = featutes
        jsondataset = json.dumps(dataset, cls=JaqpotSerializer)
        dataset_n = data_api.create_dataset_sync(self.base_url, self.api_key, jsondataset)
        print("Dataset created with id: " + dataset_n["_id"])


    # def upload_dataset(self, df=None, id=None, title=None, description=None):
    #     if title in None:
    #         raise Exception("Please submit title of the dataset")
    #     if description is None:
    #         raise Exception("Please submit description of the dataset")
    #     df_titles = list(df)
    #     for t in df_titles:
    #         type_to_c = df[t].dtypes
    #         if type_to_c == 'int64':
    #             # print(type_to_c)
    #             df[t] = df[t].astype(float)
    #     # print(df.dtypes)
    #     df = df.replace(np.nan, '', regex=True)
    #     if type(df).__name__ is not 'DataFrame':
    #         raise Exception("Cannot form a Jaqpot Dataset. Please provide a Dataframe")
    #     if id is not None:
    #         df.set_index(id, inplace=True)
    #         feat_titles = list(df)
    #         feats = []
    #         for f_t in feat_titles:
    #             fe = help.create_feature(f_t, self.user_id)
    #             jsonmi = json.dumps(fe, cls=JaqpotSerializer)
    #             # jsonmi = json.dumps(fe.__dict__)
    #             feats.append(jsonmi)
    #     else:
    #         feat_titles = list(df)
    #         feats = []
    #         for f_t in feat_titles:
    #             fe = help.create_feature(f_t, self.user_id)
    #             # jsonmi = json.dumps(fe.__dict__)
    #             jsonmi = json.dumps(fe, cls=JaqpotSerializer)
    #             feats.append(jsonmi)
    #     feat_map = {}
    #     featutes = []
    #     for feat in feats:
    #         feat_info = FeatureInfo()
    #         f = IOLoop.current().run_sync(lambda: featapi.create_feature_async(self.base_url, self.api_key, feat))
    #         feat_uri = self.base_url + "feature/" + f["_id"]
    #         feat_map[f["meta"]["titles"][0]] = feat_uri
    #         feat_info.uri = feat_uri
    #         feat_info.name = f["meta"]["titles"][0]
    #         featutes.append(feat_info.__dict__)
    #     dataset = Dataset()
    #     meta = MetaInfo()
    #     meta.creators = [self.user_id]
    #     if title is not None:
    #         meta.titles = [title]
    #     meta.descriptions = [description]
    #     dataset.meta = meta.__dict__
    #     dataset.totalRows = df.shape[0]
    #     dataset.totalColumns = df.shape[1]
    #     dataset.existence = "UPLOADED"
    #     data_entry = help.create_data_entry(df, feat_map, self.user_id)
    #     dataset.dataEntry = data_entry
    #     dataset.features = featutes
    #     jsondataset = json.dumps(dataset, cls=JaqpotSerializer)
    #     dataset_n = data_api.create_dataset_sync(self.base_url, self.api_key, jsondataset)
    #     print("Dataset created with id: " + dataset_n["_id"])

    def deploy_linear_model(self, model, X, y, title, description, algorithm):
        """
        Deploys sklearn linear_model to Jaqpot.

        Extended description of function.

        Parameters
        ----------
        model : sklearn trained model
            model is a trained model that occurs from the sklearn.linear_model family of algorithms
        X : pandas dataframe
            The dataframe that is used to train the model (X variables).
        y : pandas dataframe
            The dataframe that is used to train the model (y variables).
        title: String
            The title of the model
        description: String
            The description of the model
        algorithm: String
            The algorithm that the model implements

        Returns
        -------
        string
            The id of the model that uploaded

        """
        if isinstance(X, pd.DataFrame) is False:
            raise Exception('Function deploy_glm supports pandas dataframe or series. X is not one')
        if isinstance(y, pd.DataFrame) is False and isinstance(y, pd.Series) is False:
            raise Exception('Function deploy_glm supports pandas dataframe or series. Y is not one')
        coef_flatten = model.coef_.flatten()
        intercept_flatten = model.intercept_.flatten()
        coef_df = pd.DataFrame(list(zip(list(X), coef_flatten)), columns=['Features', 'coeff'])
        intercept = pd.DataFrame([('(intercept)', intercept_flatten[0])], columns=['Features', 'coeff'])
        coefs_all = intercept.append(coef_df, ignore_index=True)
        coefs = {}
        additionalInfo = {}
        for key in coefs_all.values:
            coefs[key[0]] = key[1]
        additionalInfo['coefficients'] = coefs
        additionalInfo['inputSeries'] = list(X)
        pretrained = help.create_pretrain_req(model, X, y, title, description,
                                              algorithm, "Linear model Scikit learn", "scikit-learn-linear-model",
                                              additionalInfo)
        # j = json.dumps(pretrained)
        j = json.dumps(pretrained, cls=JaqpotSerializer)
        response = models_api.post_pretrained_model(self.base_url, self.api_key, j)
        print("Model with id: " + response['modelId'] + " created. Please visit https://app.jaqpot.org/")

    def deploy_tree(self, model, X, y, title, description, algorithm):
        """
        Deploys sklearn tree to Jaqpot.

        Extended description of function.

        Parameters
        ----------
        model : sklearn trained model
            model is a trained model that occurs from the sklearn.tree family of algorithms
        X : pandas dataframe
            The dataframe that is used to train the model (X variables).
        y : pandas dataframe
            The dataframe that is used to train the model (y variables).
        title: String
            The title of the model
        description: String
            The description of the model
        algorithm: String
            The algorithm that the model implements

        Returns
        -------
        string
            The id of the model that uploaded

        """
        if isinstance(X, pd.DataFrame) is False:
            raise Exception('Function deploy_glm supports pandas dataframe or series. X is not one')
        if isinstance(y, pd.DataFrame) is False and isinstance(y, pd.Series) is False:
            raise Exception('Function deploy_glm supports pandas dataframe or series. Y is not one')
        additionalInfo = {}
        additionalInfo['inputSeries'] = list(X)
        pretrained = help.create_pretrain_req(model, X, y, title, description,
                                              algorithm, "Decision tree Scikit learn", "scikit-learn-tree-model",
                                              additionalInfo)
        j = json.dumps(pretrained, cls=JaqpotSerializer)
        response = models_api.post_pretrained_model(self.base_url, self.api_key, j)
        print("Model with id: " + response['modelId'] + " created. Please visit https://app.jaqpot.org/")

    def deploy_ensemble(self, model, X, y, title, description, algorithm):
        """
        Deploys sklearn ensemble to Jaqpot.

        Extended description of function.

        Parameters
        ----------
        model : sklearn trained model
            model is a trained model that occurs from the sklearn.ensemble family of algorithms
        X : pandas dataframe
            The dataframe that is used to train the model (X variables).
        y : pandas dataframe
            The dataframe that is used to train the model (y variables).
        title: String
            The title of the model
        description: String
            The description of the model
        algorithm: String
            The algorithm that the model implements

        Returns
        -------
        string
            The id of the model that uploaded

        """
        if isinstance(X, pd.DataFrame) is False:
            raise Exception('Function deploy_glm supports pandas dataframe or series. X is not one')
        if isinstance(y, pd.DataFrame) is False and isinstance(y, pd.Series) is False:
            raise Exception('Function deploy_glm supports pandas dataframe or series. Y is not one')
        additionalInfo = {}
        additionalInfo['inputSeries'] = list(X)
        pretrained = help.create_pretrain_req(model, X, y, title, description,
                                              algorithm, "Ensemble Scikit learn", "scikit-learn-ensemble-model",
                                              additionalInfo)
        j = json.dumps(pretrained, cls=JaqpotSerializer)
        response = models_api.post_pretrained_model(self.base_url, self.api_key, j)
        print("Model with id: " + response['modelId'] + " created. Please visit https://app.jaqpot.org/")

    def deploy_svm(self, model, X, y, title, description, algorithm):
        """
        Deploys sklearn svm to Jaqpot.

        Extended description of function.

        Parameters
        ----------
        model : sklearn trained model
            model is a trained model that occurs from the sklearn.svm family of algorithms
        X : pandas dataframe
            The dataframe that is used to train the model (X variables).
        y : pandas dataframe
            The dataframe that is used to train the model (y variables).
        title: String
            The title of the model
        description: String
            The description of the model
        algorithm: String
            The algorithm that the model implements

        Returns
        -------
        string
            The id of the model that uploaded

        """
        if isinstance(X, pd.DataFrame) is False:
            raise Exception('Function deploy_glm supports pandas dataframe or series. X is not one')
        if isinstance(y, pd.DataFrame) is False and isinstance(y, pd.Series) is False:
            raise Exception('Function deploy_glm supports pandas dataframe or series. Y is not one')
        additionalInfo = {}
        additionalInfo['inputSeries'] = list(X)
        pretrained = help.create_pretrain_req(model, X, y, title, description,
                                              algorithm, "Svm Scikit learn", "scikit-learn-svm-model",
                                              additionalInfo)
        j = json.dumps(pretrained, cls=JaqpotSerializer)
        response = models_api.post_pretrained_model(self.base_url, self.api_key, j)
        print("Model with id: " + response['modelId'] + " created. Please visit https://app.jaqpot.org/")

    def deploy_cluster(self, model, X, title, description, algorithm):
        """
        Deploys sklearn clustering to Jaqpot.

        Extended description of function.

        Parameters
        ----------
        model : sklearn trained model
            model is a trained model that occurs from the sklearn.svm family of algorithms
        X : pandas dataframe
            The dataframe that is used to train the model (X variables).
        y : pandas dataframe
            The dataframe that is used to train the model (y variables).
        title: String
            The title of the model
        description: String
            The description of the model
        algorithm: String
            The algorithm that the model implements

        Returns
        -------
        string
            The id of the model that uploaded

        """
        columns = ['Cluster']
        y = pd.DataFrame(columns=columns)
        if isinstance(X, pd.DataFrame) is False:
            raise Exception('Function deploy_glm supports pandas dataframe or series. X is not one')
        if isinstance(y, pd.DataFrame) is False and isinstance(y, pd.Series) is False:
            raise Exception('Function deploy_glm supports pandas dataframe or series. Y is not one')
        additionalInfo = {}
        additionalInfo['inputSeries'] = list(X)
        pretrained = help.create_pretrain_req(model, X, y, title, description,
                                              algorithm, "Svm Scikit learn", "scikit-learn-clustering-model",
                                              additionalInfo)
        j = json.dumps(pretrained, cls=JaqpotSerializer)
        response = models_api.post_pretrained_model(self.base_url, self.api_key, j)
        print("Model with id: " + response['modelId'] + " created. Please visit https://app.jaqpot.org/")

    # def deploy_biclustering(self, model, X, y, title, description, algorithm):
    #     """
    #     Deploys sklearn biclustering to Jaqpot.
    #
    #     Extended description of function.
    #
    #     Parameters
    #     ----------
    #     model : sklearn trained model
    #         model is a trained model that occurs from the sklearn.svm family of algorithms
    #     X : pandas dataframe
    #         The dataframe that is used to train the model (X variables).
    #     y : pandas dataframe
    #         The dataframe that is used to train the model (y variables).
    #     title: String
    #         The title of the model
    #     description: String
    #         The description of the model
    #     algorithm: String
    #         The algorithm that the model implements
    #
    #     Returns
    #     -------
    #     string
    #         The id of the model that uploaded
    #
    #     """
    #     if isinstance(X, pd.DataFrame) is False:
    #         raise Exception('Function deploy_glm supports pandas dataframe or series. X is not one')
    #     if isinstance(y, pd.DataFrame) is False and isinstance(y, pd.Series) is False:
    #         raise Exception('Function deploy_glm supports pandas dataframe or series. Y is not one')
    #     additionalInfo = {}
    #     additionalInfo['inputSeries'] = list(X)
    #     pretrained = help.create_pretrain_req(model, X, y, title, description,
    #                                           algorithm, "Svm Scikit learn", "scikit-learn-biclustering-model",
    #                                           additionalInfo)
    #     j = json.dumps(pretrained, cls=JaqpotSerializer)
    #     response = models_api.post_pretrained_model(self.base_url, self.api_key, j)
    #     print("Model with id: " + response['modelId'] + " created. Please visit https://app.jaqpot.org/")

    def deploy_naive_bayess(self, model, X, y, title, description, algorithm):
        """
        Deploys sklearn naive_bayess to Jaqpot.

        Extended description of function.

        Parameters
        ----------
        model : sklearn trained model
            model is a trained model that occurs from the sklearn.svm family of algorithms
        X : pandas dataframe
            The dataframe that is used to train the model (X variables).
        y : pandas dataframe
            The dataframe that is used to train the model (y variables).
        title: String
            The title of the model
        description: String
            The description of the model
        algorithm: String
            The algorithm that the model implements

        Returns
        -------
        string
            The id of the model that uploaded

        """
        if isinstance(X, pd.DataFrame) is False:
            raise Exception('Function deploy_glm supports pandas dataframe or series. X is not one')
        if isinstance(y, pd.DataFrame) is False and isinstance(y, pd.Series) is False:
            raise Exception('Function deploy_glm supports pandas dataframe or series. Y is not one')
        additionalInfo = {}
        additionalInfo['inputSeries'] = list(X)
        pretrained = help.create_pretrain_req(model, X, y, title, description,
                                              algorithm, "Svm Scikit learn", "scikit-learn-naive-bayess-model",
                                              additionalInfo)
        j = json.dumps(pretrained, cls=JaqpotSerializer)
        response = models_api.post_pretrained_model(self.base_url, self.api_key, j)
        print("Model with id: " + response['modelId'] + " created. Please visit https://app.jaqpot.org/")

    def deploy_nearest_neighbors(self, model, X, y, title, description, algorithm):
        """
        Deploys sklearn naive_neighbours to Jaqpot.

        Extended description of function.

        Parameters
        ----------
        model : sklearn trained model
            model is a trained model that occurs from the sklearn.svm family of algorithms
        X : pandas dataframe
            The dataframe that is used to train the model (X variables).
        y : pandas dataframe
            The dataframe that is used to train the model (y variables).
        title: String
            The title of the model
        description: String
            The description of the model
        algorithm: String
            The algorithm that the model implements

        Returns
        -------
        string
            The id of the model that uploaded

        """
        if isinstance(X, pd.DataFrame) is False:
            raise Exception('Function deploy_glm supports pandas dataframe or series. X is not one')
        if isinstance(y, pd.DataFrame) is False and isinstance(y, pd.Series) is False:
            raise Exception('Function deploy_glm supports pandas dataframe or series. Y is not one')
        additionalInfo = {}
        additionalInfo['inputSeries'] = list(X)
        pretrained = help.create_pretrain_req(model, X, y, title, description,
                                              algorithm, "Svm Scikit learn", "scikit-learn-nearest-neighbours-model",
                                              additionalInfo)
        j = json.dumps(pretrained, cls=JaqpotSerializer)
        response = models_api.post_pretrained_model(self.base_url, self.api_key, j)
        print("Model with id: " + response['modelId'] + " created. Please visit https://app.jaqpot.org/")

    def deploy_neural_network(self, model, X, y, title, description, algorithm):
        """
        Deploys sklearn neural_network to Jaqpot.

        Extended description of function.

        Parameters
        ----------
        model : sklearn trained model
            model is a trained model that occurs from the sklearn.svm family of algorithms
        X : pandas dataframe
            The dataframe that is used to train the model (X variables).
        y : pandas dataframe
            The dataframe that is used to train the model (y variables).
        title: String
            The title of the model
        description: String
            The description of the model
        algorithm: String
            The algorithm that the model implements

        Returns
        -------
        string
            The id of the model that uploaded

        """
        if isinstance(X, pd.DataFrame) is False:
            raise Exception('Function deploy_glm supports pandas dataframe or series. X is not one')
        if isinstance(y, pd.DataFrame) is False and isinstance(y, pd.Series) is False:
            raise Exception('Function deploy_glm supports pandas dataframe or series. Y is not one')
        additionalInfo = {}
        additionalInfo['inputSeries'] = list(X)
        pretrained = help.create_pretrain_req(model, X, y, title, description,
                                              algorithm, "Svm Scikit learn", "scikit-learn-neural-network-model",
                                              additionalInfo)
        j = json.dumps(pretrained, cls=JaqpotSerializer)
        response = models_api.post_pretrained_model(self.base_url, self.api_key, j)
        print("Model with id: " + response['modelId'] + " created. Please visit https://app.jaqpot.org/")

    def api_key(self):
        return self.api_key

    def check_key(self):
        if self.api_key is None:
            raise Exception("Api key not present!")



from .meta import MetaInfo  # noqa: F401,E501


class ErrorReport(object):

    def __init__(self, meta=None, ontological_classes=None, visible=False, temporary=False, featured=False, code=None, actor=None, message=None, details=None, http_status=None, id=None):

        self._meta = None
        self._ontological_classes = None
        self._visible = None
        self._temporary = None
        self._featured = None
        self._code = None
        self._actor = None
        self._message = None
        self._details = None
        self._http_status = None
        self._id = None
        self.discriminator = None

        if meta is not None:
            self.meta = meta
        if ontological_classes is not None:
            self.ontological_classes = ontological_classes
        if visible is not None:
            self.visible = visible
        if temporary is not None:
            self.temporary = temporary
        if featured is not None:
            self.featured = featured
        if code is not None:
            self.code = code
        if actor is not None:
            self.actor = actor
        if message is not None:
            self.message = message
        if details is not None:
            self.details = details
        if http_status is not None:
            self.http_status = http_status
        if id is not None:
            self.id = id

    @property
    def meta(self):
        """Gets the meta of this ErrorReport.  # noqa: E501


        :return: The meta of this ErrorReport.  # noqa: E501
        :rtype: MetaInfo
        """
        return self._meta

    @meta.setter
    def meta(self, meta):
        """Sets the meta of this ErrorReport.


        :param meta: The meta of this ErrorReport.  # noqa: E501
        :type: MetaInfo
        """

        self._meta = meta

    @property
    def ontological_classes(self):
        """Gets the ontological_classes of this ErrorReport.  # noqa: E501


        :return: The ontological_classes of this ErrorReport.  # noqa: E501
        :rtype: list[str]
        """
        return self._ontological_classes

    @ontological_classes.setter
    def ontological_classes(self, ontological_classes):
        """Sets the ontological_classes of this ErrorReport.


        :param ontological_classes: The ontological_classes of this ErrorReport.  # noqa: E501
        :type: list[str]
        """

        self._ontological_classes = ontological_classes

    @property
    def visible(self):
        """Gets the visible of this ErrorReport.  # noqa: E501


        :return: The visible of this ErrorReport.  # noqa: E501
        :rtype: bool
        """
        return self._visible

    @visible.setter
    def visible(self, visible):
        """Sets the visible of this ErrorReport.


        :param visible: The visible of this ErrorReport.  # noqa: E501
        :type: bool
        """

        self._visible = visible

    @property
    def temporary(self):
        """Gets the temporary of this ErrorReport.  # noqa: E501


        :return: The temporary of this ErrorReport.  # noqa: E501
        :rtype: bool
        """
        return self._temporary

    @temporary.setter
    def temporary(self, temporary):
        """Sets the temporary of this ErrorReport.


        :param temporary: The temporary of this ErrorReport.  # noqa: E501
        :type: bool
        """

        self._temporary = temporary

    @property
    def featured(self):
        """Gets the featured of this ErrorReport.  # noqa: E501


        :return: The featured of this ErrorReport.  # noqa: E501
        :rtype: bool
        """
        return self._featured

    @featured.setter
    def featured(self, featured):
        """Sets the featured of this ErrorReport.


        :param featured: The featured of this ErrorReport.  # noqa: E501
        :type: bool
        """

        self._featured = featured

    @property
    def code(self):
        """Gets the code of this ErrorReport.  # noqa: E501

        Error code  # noqa: E501

        :return: The code of this ErrorReport.  # noqa: E501
        :rtype: str
        """
        return self._code

    @code.setter
    def code(self, code):
        """Sets the code of this ErrorReport.

        Error code  # noqa: E501

        :param code: The code of this ErrorReport.  # noqa: E501
        :type: str
        """

        self._code = code

    @property
    def actor(self):
        """Gets the actor of this ErrorReport.  # noqa: E501

        Who is to blame  # noqa: E501

        :return: The actor of this ErrorReport.  # noqa: E501
        :rtype: str
        """
        return self._actor

    @actor.setter
    def actor(self, actor):
        """Sets the actor of this ErrorReport.

        Who is to blame  # noqa: E501

        :param actor: The actor of this ErrorReport.  # noqa: E501
        :type: str
        """

        self._actor = actor

    @property
    def message(self):
        """Gets the message of this ErrorReport.  # noqa: E501

        Short error message  # noqa: E501

        :return: The message of this ErrorReport.  # noqa: E501
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """Sets the message of this ErrorReport.

        Short error message  # noqa: E501

        :param message: The message of this ErrorReport.  # noqa: E501
        :type: str
        """

        self._message = message

    @property
    def details(self):
        """Gets the details of this ErrorReport.  # noqa: E501

        Details to be used for debugging.  # noqa: E501

        :return: The details of this ErrorReport.  # noqa: E501
        :rtype: str
        """
        return self._details

    @details.setter
    def details(self, details):
        """Sets the details of this ErrorReport.

        Details to be used for debugging.  # noqa: E501

        :param details: The details of this ErrorReport.  # noqa: E501
        :type: str
        """

        self._details = details

    @property
    def http_status(self):
        """Gets the http_status of this ErrorReport.  # noqa: E501

        Accompanying HTTP status.  # noqa: E501

        :return: The http_status of this ErrorReport.  # noqa: E501
        :rtype: int
        """
        return self._http_status

    @http_status.setter
    def http_status(self, http_status):
        """Sets the http_status of this ErrorReport.

        Accompanying HTTP status.  # noqa: E501

        :param http_status: The http_status of this ErrorReport.  # noqa: E501
        :type: int
        """

        self._http_status = http_status

    @property
    def id(self):
        """Gets the id of this ErrorReport.  # noqa: E501


        :return: The id of this ErrorReport.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ErrorReport.


        :param id: The id of this ErrorReport.  # noqa: E501
        :type: str
        """

        self._id = id



class FeatureInfo(object):

    def __init__(self, name=None, units=None, conditions=None, category=None, ont=None, uri=None):

        self.name = None
        self.units = None
        self.conditions = None
        self.category = None
        # self.ont = None
        self.uri = None

        if name is not None:
            self.name = name
        if units is not None:
            self.units = units
        if conditions is not None:
            self.conditions = conditions
        if category is not None:
            self.category = category
        # if ont is not None:
        #     self.ont = ont
        if uri is not None:
            self.uri = uri

    # @property
    # def name(self):
    #     """Gets the name of this FeatureInfo.  # noqa: E501
    #
    #
    #     :return: The name of this FeatureInfo.  # noqa: E501
    #     :rtype: str
    #     """
    #     return self.name
    #
    # @name.setter
    # def name(self, name):
    #     """Sets the name of this FeatureInfo.
    #
    #
    #     :param name: The name of this FeatureInfo.  # noqa: E501
    #     :type: str
    #     """
    #
    #     self.name = name
    #
    # @property
    # def units(self):
    #     """Gets the units of this FeatureInfo.  # noqa: E501
    #
    #
    #     :return: The units of this FeatureInfo.  # noqa: E501
    #     :rtype: str
    #     """
    #     return self.units
    #
    # @units.setter
    # def units(self, units):
    #     """Sets the units of this FeatureInfo.
    #
    #
    #     :param units: The units of this FeatureInfo.  # noqa: E501
    #     :type: str
    #     """
    #
    #     self.units = units
    #
    # @property
    # def conditions(self):
    #     """Gets the conditions of this FeatureInfo.  # noqa: E501
    #
    #
    #     :return: The conditions of this FeatureInfo.  # noqa: E501
    #     :rtype: dict(str, object)
    #     """
    #     return self.conditions
    #
    # @conditions.setter
    # def conditions(self, conditions):
    #     """Sets the conditions of this FeatureInfo.
    #
    #
    #     :param conditions: The conditions of this FeatureInfo.  # noqa: E501
    #     :type: dict(str, object)
    #     """
    #
    #     self.conditions = conditions
    #
    # @property
    # def category(self):
    #     """Gets the category of this FeatureInfo.  # noqa: E501
    #
    #
    #     :return: The category of this FeatureInfo.  # noqa: E501
    #     :rtype: str
    #     """
    #     return self.category
    #
    # @category.setter
    # def category(self, category):
    #     """Sets the category of this FeatureInfo.
    #
    #
    #     :param category: The category of this FeatureInfo.  # noqa: E501
    #     :type: str
    #     """
    #     allowedvalues = ["EXPERIMENTAL", "IMAGE", "GO", "MOPAC", "CDK", "PREDICTED"]
    #     if category not in allowedvalues:
    #         raise ValueError(
    #             "Invalid value for `category` ({0}), must be one of {1}"
    #             .format(category, allowedvalues)
    #         )
    #
    #     self.category = category
    #
    # @property
    # def ont(self):
    #     """Gets the ont of this FeatureInfo.  # noqa: E501
    #
    #
    #     :return: The ont of this FeatureInfo.  # noqa: E501
    #     :rtype: str
    #     """
    #     return self.ont
    #
    # @ont.setter
    # def ont(self, ont):
    #     """Sets the ont of this FeatureInfo.
    #
    #
    #     :param ont: The ont of this FeatureInfo.  # noqa: E501
    #     :type: str
    #     """
    #
    #     self.ont = ont
    #
    # @property
    # def uri(self):
    #     """Gets the uri of this FeatureInfo.  # noqa: E501
    #
    #
    #     :return: The uri of this FeatureInfo.  # noqa: E501
    #     :rtype: str
    #     """
    #     return self.uri
    #
    # @uri.setter
    # def uri(self, uri):
    #     """Sets the uri of this FeatureInfo.
    #
    #
    #     :param uri: The uri of this FeatureInfo.  # noqa: E501
    #     :type: str
    #     """
    #
    #     self.uri = uri



class PretrainedModel(object):

    def __init__(self, raw_model=None, pmml_model=None, additional_info=None, dependent_features=None, independent_features=None, predicted_features=None, implemented_in=None, implemented_with=None, title=None, discription=None, algorithm=None):

        self._raw_model = None
        self._pmml_model = None
        self._additional_info = None
        self._dependent_features = None
        self._independent_features = None
        self._predicted_features = None
        self._implemented_in = None
        self._implemented_with = None
        self._title = None
        self._discription = None
        self._algorithm = None
        self.discriminator = None

        if raw_model is not None:
            self.raw_model = raw_model
        if pmml_model is not None:
            self.pmml_model = pmml_model
        if additional_info is not None:
            self.additional_info = additional_info
        if dependent_features is not None:
            self.dependent_features = dependent_features
        if independent_features is not None:
            self.independent_features = independent_features
        if predicted_features is not None:
            self.predicted_features = predicted_features
        if implemented_in is not None:
            self.implemented_in = implemented_in
        if implemented_with is not None:
            self.implemented_with = implemented_with
        if title is not None:
            self.title = title
        if discription is not None:
            self.discription = discription
        if algorithm is not None:
            self.algorithm = algorithm

    @property
    def raw_model(self):
        """Gets the raw_model of this PretrainedModel.  # noqa: E501


        :return: The raw_model of this PretrainedModel.  # noqa: E501
        :rtype: object
        """
        return self._raw_model

    @raw_model.setter
    def raw_model(self, raw_model):
        """Sets the raw_model of this PretrainedModel.


        :param raw_model: The raw_model of this PretrainedModel.  # noqa: E501
        :type: object
        """

        self._raw_model = raw_model

    @property
    def pmml_model(self):
        """Gets the pmml_model of this PretrainedModel.  # noqa: E501


        :return: The pmml_model of this PretrainedModel.  # noqa: E501
        :rtype: object
        """
        return self._pmml_model

    @pmml_model.setter
    def pmml_model(self, pmml_model):
        """Sets the pmml_model of this PretrainedModel.


        :param pmml_model: The pmml_model of this PretrainedModel.  # noqa: E501
        :type: object
        """

        self._pmml_model = pmml_model

    @property
    def additional_info(self):
        """Gets the additional_info of this PretrainedModel.  # noqa: E501


        :return: The additional_info of this PretrainedModel.  # noqa: E501
        :rtype: object
        """
        return self._additional_info

    @additional_info.setter
    def additional_info(self, additional_info):
        """Sets the additional_info of this PretrainedModel.


        :param additional_info: The additional_info of this PretrainedModel.  # noqa: E501
        :type: object
        """

        self._additional_info = additional_info

    @property
    def dependent_features(self):
        """Gets the dependent_features of this PretrainedModel.  # noqa: E501


        :return: The dependent_features of this PretrainedModel.  # noqa: E501
        :rtype: list[str]
        """
        return self._dependent_features

    @dependent_features.setter
    def dependent_features(self, dependent_features):
        """Sets the dependent_features of this PretrainedModel.


        :param dependent_features: The dependent_features of this PretrainedModel.  # noqa: E501
        :type: list[str]
        """

        self._dependent_features = dependent_features

    @property
    def independent_features(self):
        """Gets the independent_features of this PretrainedModel.  # noqa: E501


        :return: The independent_features of this PretrainedModel.  # noqa: E501
        :rtype: list[str]
        """
        return self._independent_features

    @independent_features.setter
    def independent_features(self, independent_features):
        """Sets the independent_features of this PretrainedModel.


        :param independent_features: The independent_features of this PretrainedModel.  # noqa: E501
        :type: list[str]
        """

        self._independent_features = independent_features

    @property
    def predicted_features(self):
        """Gets the predicted_features of this PretrainedModel.  # noqa: E501


        :return: The predicted_features of this PretrainedModel.  # noqa: E501
        :rtype: list[str]
        """
        return self._predicted_features

    @predicted_features.setter
    def predicted_features(self, predicted_features):
        """Sets the predicted_features of this PretrainedModel.


        :param predicted_features: The predicted_features of this PretrainedModel.  # noqa: E501
        :type: list[str]
        """

        self._predicted_features = predicted_features

    @property
    def implemented_in(self):
        """Gets the implemented_in of this PretrainedModel.  # noqa: E501


        :return: The implemented_in of this PretrainedModel.  # noqa: E501
        :rtype: list[str]
        """
        return self._implemented_in

    @implemented_in.setter
    def implemented_in(self, implemented_in):
        """Sets the implemented_in of this PretrainedModel.


        :param implemented_in: The implemented_in of this PretrainedModel.  # noqa: E501
        :type: list[str]
        """

        self._implemented_in = implemented_in

    @property
    def implemented_with(self):
        """Gets the implemented_with of this PretrainedModel.  # noqa: E501


        :return: The implemented_with of this PretrainedModel.  # noqa: E501
        :rtype: list[str]
        """
        return self._implemented_with

    @implemented_with.setter
    def implemented_with(self, implemented_with):
        """Sets the implemented_with of this PretrainedModel.


        :param implemented_with: The implemented_with of this PretrainedModel.  # noqa: E501
        :type: list[str]
        """

        self._implemented_with = implemented_with

    @property
    def title(self):
        """Gets the title of this PretrainedModel.  # noqa: E501


        :return: The title of this PretrainedModel.  # noqa: E501
        :rtype: list[str]
        """
        return self._title

    @title.setter
    def title(self, title):
        """Sets the title of this PretrainedModel.


        :param title: The title of this PretrainedModel.  # noqa: E501
        :type: list[str]
        """

        self._title = title

    @property
    def discription(self):
        """Gets the discription of this PretrainedModel.  # noqa: E501


        :return: The discription of this PretrainedModel.  # noqa: E501
        :rtype: list[str]
        """
        return self._discription

    @discription.setter
    def discription(self, discription):
        """Sets the discription of this PretrainedModel.


        :param discription: The discription of this PretrainedModel.  # noqa: E501
        :type: list[str]
        """

        self._discription = discription

    @property
    def algorithm(self):
        """Gets the algorithm of this PretrainedModel.  # noqa: E501


        :return: The algorithm of this PretrainedModel.  # noqa: E501
        :rtype: list[str]
        """
        return self._algorithm

    @algorithm.setter
    def algorithm(self, algorithm):
        """Sets the algorithm of this PretrainedModel.


        :param algorithm: The algorithm of this PretrainedModel.  # noqa: E501
        :type: list[str]
        """

        self._algorithm = algorithm


class EntryId(object):

    def __init__(self, name=None, ownerUUID=None, URI=None, type=None):
        self.name = None
        self.ownerUUID = None
        self.URI = None
        self.type = None

        if name is not None:
            self.name = name
        if ownerUUID is not None:
            self.owner_uuid = ownerUUID
        if URI is not None:
            self.URI = URI
        if type is not None:
            self.type = type

    # @property
    # def name(self):
    #     """Gets the name of this EntryId.  # noqa: E501
    #
    #
    #     :return: The name of this EntryId.  # noqa: E501
    #     :rtype: str
    #     """
    #     return self._name
    #
    # @name.setter
    # def name(self, name):
    #     """Sets the name of this EntryId.
    #
    #
    #     :param name: The name of this EntryId.  # noqa: E501
    #     :type: str
    #     """
    #
    #     self._name = name
    #
    # @property
    # def owner_uuid(self):
    #     """Gets the owner_uuid of this EntryId.  # noqa: E501
    #
    #
    #     :return: The owner_uuid of this EntryId.  # noqa: E501
    #     :rtype: str
    #     """
    #     return self._owner_uuid
    #
    # @owner_uuid.setter
    # def owner_uuid(self, owner_uuid):
    #     """Sets the owner_uuid of this EntryId.
    #
    #
    #     :param owner_uuid: The owner_uuid of this EntryId.  # noqa: E501
    #     :type: str
    #     """
    #
    #     self._owner_uuid = owner_uuid
    #
    # @property
    # def uri(self):
    #     """Gets the uri of this EntryId.  # noqa: E501
    #
    #
    #     :return: The uri of this EntryId.  # noqa: E501
    #     :rtype: str
    #     """
    #     return self._uri
    #
    # @uri.setter
    # def uri(self, uri):
    #     """Sets the uri of this EntryId.
    #
    #
    #     :param uri: The uri of this EntryId.  # noqa: E501
    #     :type: str
    #     """
    #
    #     self._uri = uri



import json
import jaqpotpy.helpers.helpers as hel
import numpy as np


class JaqpotSerializer(json.JSONEncoder):

    # def __init__(self, *args, **kwargs):
    #     json.JSONEncoder.__init__(self, object_hook=self.object_hook, *args, **kwargs)

    def default(self, o):
        # if isinstance(o, np.ndarray):
        #     if o.flags['C_CONTIGUOUS']:
        #         obj_data = o.data
        #     else:
        #         cont_obj = np.ascontiguousarray(o)
        #         assert (cont_obj.flags['C_CONTIGUOUS'])
        #         obj_data = cont_obj.data
        #     return o.tolist()
        delete = []
        for key in o.__dict__:
            if getattr(o, key) is None or "":
                delete.append(key)
        for keytod in delete:
            delattr(o, keytod)
        return o.__dict__



# class JaqpotDeserializer(json.JSONDecoder):
#
#     def __init__(self, *args, **kwargs):
#         json.JSONDecoder.__init__(self, object_hook=self.object_hook, *args, **kwargs)
#
#     def object_hook(self, obj):
#         print("From des " + str(obj))
#         # id = obj["_id"]
#         # print(id)
#         # print(getattr(obj, "_id"))
#         # setattr()
#         return obj




# fe = hel.create_feature("sadf", "adf")

# fe1 = hel.clear_entity(fe.__dict__)
# print(fe1)

# print(fe.__dict__)
# print(json.dumps(fe.__dict__))
# print(json.dumps(fe, cls=JaqpotSerializer))
# print(json.dumps(fe.__dict__, cls=JaqpotSerializer))



from tornado.web import RequestHandler


class BaseClient(RequestHandler):

    def get(self, *args, **kwargs):
        self.write("say something")

    def set_default_headers(self, *args, **kwargs):
        self.set_header("Access-Control-Allow-Origin", "*")
        self.set_header("Access-Control-Allow-Headers", "x-requested-with")
        self.set_header("Access-Control-Allow-Methods", "POST, GET, OPTIONS")


from .jaqpot_base import JaqpotEntity
from .dataentry import DataEntry
from .featureinfo import FeatureInfo
from .meta import MetaInfo


class Dataset(object):

    def __init__(self, meta=None, ontologicalClasses=None, visible=False, temporary=False, featured=False, datasetUri=None, byModel=None, dataEntry=None, features=None, totalRows=None, totalColumns=None, descriptors=None, id=None, existence=None):
        """Dataset - a model defined in Swagger"""  # noqa: E501

        self.meta = None
        self.ontologicalClasses = None
        self.visible = None
        self.temporary = None
        self.featured = None
        self.datasetUri = None
        self.byModel = None
        self.dataEntry = None
        self.features = None
        self.totalRows = None
        self.totalColumns = None
        self.descriptors = None
        self.id = None
        self.existence = None

        if meta is not None:
            self.meta = meta
        if ontologicalClasses is not None:
            self.ontologicalClasses = ontologicalClasses
        if visible is not None:
            self.visible = visible
        if temporary is not None:
            self.temporary = temporary
        if featured is not None:
            self.featured = featured
        if datasetUri is not None:
            self.dataset_uri = datasetUri
        if byModel is not None:
            self.by_model = byModel
        if dataEntry is not None:
            self.data_entry = dataEntry
        if features is not None:
            self.features = features
        if totalRows is not None:
            self.totalRows = totalRows
        if totalColumns is not None:
            self.totalColumns = totalColumns
        if descriptors is not None:
            self.descriptors = descriptors
        if id is not None:
            self.id = id
        if existence is not None:
            self.existence = existence

    # @property
    # def meta(self):
    #     """Gets the meta of this Dataset.  # noqa: E501
    #
    #
    #     :return: The meta of this Dataset.  # noqa: E501
    #     :rtype: MetaInfo
    #     """
    #     return self.meta
    #
    # @meta.setter
    # def meta(self, meta):
    #     """Sets the meta of this Dataset.
    #
    #
    #     :param meta: The meta of this Dataset.  # noqa: E501
    #     :type: MetaInfo
    #     """
    #
    #     self.meta = meta
    #
    # @property
    # def ontological_classes(self):
    #     """Gets the ontological_classes of this Dataset.  # noqa: E501
    #
    #
    #     :return: The ontological_classes of this Dataset.  # noqa: E501
    #     :rtype: list[str]
    #     """
    #     return self.ontological_classes
    #
    # @ontological_classes.setter
    # def ontological_classes(self, ontological_classes):
    #     """Sets the ontological_classes of this Dataset.
    #
    #
    #     :param ontological_classes: The ontological_classes of this Dataset.  # noqa: E501
    #     :type: list[str]
    #     """
    #
    #     self.ontological_classes = ontological_classes
    #
    # @property
    # def visible(self):
    #     """Gets the visible of this Dataset.  # noqa: E501
    #
    #
    #     :return: The visible of this Dataset.  # noqa: E501
    #     :rtype: bool
    #     """
    #     return self.visible
    #
    # @visible.setter
    # def visible(self, visible):
    #     """Sets the visible of this Dataset.
    #
    #
    #     :param visible: The visible of this Dataset.  # noqa: E501
    #     :type: bool
    #     """
    #
    #     self.visible = visible
    #
    # @property
    # def temporary(self):
    #     """Gets the temporary of this Dataset.  # noqa: E501
    #
    #
    #     :return: The temporary of this Dataset.  # noqa: E501
    #     :rtype: bool
    #     """
    #     return self.temporary
    #
    # @temporary.setter
    # def temporary(self, temporary):
    #     """Sets the temporary of this Dataset.
    #
    #
    #     :param temporary: The temporary of this Dataset.  # noqa: E501
    #     :type: bool
    #     """
    #
    #     self.temporary = temporary
    #
    # @property
    # def featured(self):
    #     """Gets the featured of this Dataset.  # noqa: E501
    #
    #
    #     :return: The featured of this Dataset.  # noqa: E501
    #     :rtype: bool
    #     """
    #     return self.featured
    #
    # @featured.setter
    # def featured(self, featured):
    #     """Sets the featured of this Dataset.
    #
    #
    #     :param featured: The featured of this Dataset.  # noqa: E501
    #     :type: bool
    #     """
    #
    #     self.featured = featured
    #
    # @property
    # def dataset_uri(self):
    #     """Gets the dataset_uri of this Dataset.  # noqa: E501
    #
    #
    #     :return: The dataset_uri of this Dataset.  # noqa: E501
    #     :rtype: str
    #     """
    #     return self.dataset_uri
    #
    # @dataset_uri.setter
    # def dataset_uri(self, dataset_uri):
    #     """Sets the dataset_uri of this Dataset.
    #
    #
    #     :param dataset_uri: The dataset_uri of this Dataset.  # noqa: E501
    #     :type: str
    #     """
    #
    #     self.dataset_uri = dataset_uri
    #
    # @property
    # def by_model(self):
    #     """Gets the by_model of this Dataset.  # noqa: E501
    #
    #
    #     :return: The by_model of this Dataset.  # noqa: E501
    #     :rtype: str
    #     """
    #     return self.by_model
    #
    # @by_model.setter
    # def by_model(self, by_model):
    #     """Sets the by_model of this Dataset.
    #
    #
    #     :param by_model: The by_model of this Dataset.  # noqa: E501
    #     :type: str
    #     """
    #
    #     self.by_model = by_model
    #
    # @property
    # def data_entry(self):
    #     """Gets the data_entry of this Dataset.  # noqa: E501
    #
    #
    #     :return: The data_entry of this Dataset.  # noqa: E501
    #     :rtype: list[DataEntry]
    #     """
    #     return self.data_entry
    #
    # @data_entry.setter
    # def data_entry(self, data_entry):
    #     """Sets the data_entry of this Dataset.
    #
    #
    #     :param data_entry: The data_entry of this Dataset.  # noqa: E501
    #     :type: list[DataEntry]
    #     """
    #
    #     self.data_entry = data_entry
    #
    # @property
    # def features(self):
    #     """Gets the features of this Dataset.  # noqa: E501
    #
    #
    #     :return: The features of this Dataset.  # noqa: E501
    #     :rtype: list[FeatureInfo]
    #     """
    #     return self.features
    #
    # @features.setter
    # def features(self, features):
    #     """Sets the features of this Dataset.
    #
    #
    #     :param features: The features of this Dataset.  # noqa: E501
    #     :type: list[FeatureInfo]
    #     """
    #
    #     self.features = features
    #
    # @property
    # def total_rows(self):
    #     """Gets the total_rows of this Dataset.  # noqa: E501
    #
    #
    #     :return: The total_rows of this Dataset.  # noqa: E501
    #     :rtype: int
    #     """
    #     return self.total_rows
    #
    # @total_rows.setter
    # def total_rows(self, total_rows):
    #     """Sets the total_rows of this Dataset.
    #
    #
    #     :param total_rows: The total_rows of this Dataset.  # noqa: E501
    #     :type: int
    #     """
    #
    #     self.total_rows = total_rows
    #
    # @property
    # def total_columns(self):
    #     """Gets the total_columns of this Dataset.  # noqa: E501
    #
    #
    #     :return: The total_columns of this Dataset.  # noqa: E501
    #     :rtype: int
    #     """
    #     return self.total_columns
    #
    # @total_columns.setter
    # def total_columns(self, total_columns):
    #     """Sets the total_columns of this Dataset.
    #
    #
    #     :param total_columns: The total_columns of this Dataset.  # noqa: E501
    #     :type: int
    #     """
    #
    #     self.total_columns = total_columns
    #
    # @property
    # def descriptors(self):
    #     """Gets the descriptors of this Dataset.  # noqa: E501
    #
    #
    #     :return: The descriptors of this Dataset.  # noqa: E501
    #     :rtype: list[str]
    #     """
    #     return self.descriptors
    #
    # @descriptors.setter
    # def descriptors(self, descriptors):
    #     """Sets the descriptors of this Dataset.
    #
    #
    #     :param descriptors: The descriptors of this Dataset.  # noqa: E501
    #     :type: list[str]
    #     """
    #     allowed_values = ["EXPERIMENTAL", "IMAGE", "GO", "MOPAC", "CDK", "PREDICTED"]  # noqa: E501
    #     if not set(descriptors).issubset(set(allowed_values)):
    #         raise ValueError(
    #             "Invalid values for `descriptors` [{0}], must be a subset of [{1}]"  # noqa: E501
    #             .format(", ".join(map(str, set(descriptors) - set(allowed_values))),  # noqa: E501
    #                     ", ".join(map(str, allowed_values)))
    #         )
    #
    #     self._descriptors = descriptors
    #
    # @property
    # def id(self):
    #     """Gets the id of this Dataset.  # noqa: E501
    #
    #
    #     :return: The id of this Dataset.  # noqa: E501
    #     :rtype: str
    #     """
    #     return self.id
    #
    # @id.setter
    # def id(self, id):
    #     """Sets the id of this Dataset.
    #
    #
    #     :param id: The id of this Dataset.  # noqa: E501
    #     :type: str
    #     """
    #
    #     self.id = id


from .error_report import ErrorReport  # noqa: F401,E501
from .meta import MetaInfo  # noqa: F401,E501


class Task(object):

    def __init__(self, meta=None, ontological_classes=None, visible=False, temporary=False, featured=False, result_uri=None, result=None, percentage_completed=None, error_report=None, http_status=None, duration=None, type=None, id=None, status=None):  # noqa: E501
        """Task - a model defined in Swagger"""  # noqa: E501

        self._meta = None
        self._ontological_classes = None
        self._visible = None
        self._temporary = None
        self._featured = None
        self._result_uri = None
        self._result = None
        self._percentage_completed = None
        self._error_report = None
        self._http_status = None
        self._duration = None
        self._type = None
        self._id = None
        self._status = None
        self.discriminator = None

        if meta is not None:
            self.meta = meta
        if ontological_classes is not None:
            self.ontological_classes = ontological_classes
        if visible is not None:
            self.visible = visible
        if temporary is not None:
            self.temporary = temporary
        if featured is not None:
            self.featured = featured
        if result_uri is not None:
            self.result_uri = result_uri
        if result is not None:
            self.result = result
        if percentage_completed is not None:
            self.percentage_completed = percentage_completed
        if error_report is not None:
            self.error_report = error_report
        if http_status is not None:
            self.http_status = http_status
        if duration is not None:
            self.duration = duration
        if type is not None:
            self.type = type
        if id is not None:
            self.id = id
        if status is not None:
            self.status = status

    @property
    def meta(self):
        """Gets the meta of this Task.  # noqa: E501


        :return: The meta of this Task.  # noqa: E501
        :rtype: MetaInfo
        """
        return self._meta

    @meta.setter
    def meta(self, meta):
        """Sets the meta of this Task.


        :param meta: The meta of this Task.  # noqa: E501
        :type: MetaInfo
        """

        self._meta = meta

    @property
    def ontological_classes(self):
        """Gets the ontological_classes of this Task.  # noqa: E501


        :return: The ontological_classes of this Task.  # noqa: E501
        :rtype: list[str]
        """
        return self._ontological_classes

    @ontological_classes.setter
    def ontological_classes(self, ontological_classes):
        """Sets the ontological_classes of this Task.


        :param ontological_classes: The ontological_classes of this Task.  # noqa: E501
        :type: list[str]
        """

        self._ontological_classes = ontological_classes

    @property
    def visible(self):
        """Gets the visible of this Task.  # noqa: E501


        :return: The visible of this Task.  # noqa: E501
        :rtype: bool
        """
        return self._visible

    @visible.setter
    def visible(self, visible):
        """Sets the visible of this Task.


        :param visible: The visible of this Task.  # noqa: E501
        :type: bool
        """

        self._visible = visible

    @property
    def temporary(self):
        """Gets the temporary of this Task.  # noqa: E501


        :return: The temporary of this Task.  # noqa: E501
        :rtype: bool
        """
        return self._temporary

    @temporary.setter
    def temporary(self, temporary):
        """Sets the temporary of this Task.


        :param temporary: The temporary of this Task.  # noqa: E501
        :type: bool
        """

        self._temporary = temporary

    @property
    def featured(self):
        """Gets the featured of this Task.  # noqa: E501


        :return: The featured of this Task.  # noqa: E501
        :rtype: bool
        """
        return self._featured

    @featured.setter
    def featured(self, featured):
        """Sets the featured of this Task.


        :param featured: The featured of this Task.  # noqa: E501
        :type: bool
        """

        self._featured = featured

    @property
    def result_uri(self):
        """Gets the result_uri of this Task.  # noqa: E501


        :return: The result_uri of this Task.  # noqa: E501
        :rtype: str
        """
        return self._result_uri

    @result_uri.setter
    def result_uri(self, result_uri):
        """Sets the result_uri of this Task.


        :param result_uri: The result_uri of this Task.  # noqa: E501
        :type: str
        """

        self._result_uri = result_uri

    @property
    def result(self):
        """Gets the result of this Task.  # noqa: E501


        :return: The result of this Task.  # noqa: E501
        :rtype: str
        """
        return self._result

    @result.setter
    def result(self, result):
        """Sets the result of this Task.


        :param result: The result of this Task.  # noqa: E501
        :type: str
        """

        self._result = result

    @property
    def percentage_completed(self):
        """Gets the percentage_completed of this Task.  # noqa: E501


        :return: The percentage_completed of this Task.  # noqa: E501
        :rtype: float
        """
        return self._percentage_completed

    @percentage_completed.setter
    def percentage_completed(self, percentage_completed):
        """Sets the percentage_completed of this Task.


        :param percentage_completed: The percentage_completed of this Task.  # noqa: E501
        :type: float
        """

        self._percentage_completed = percentage_completed

    @property
    def error_report(self):
        """Gets the error_report of this Task.  # noqa: E501


        :return: The error_report of this Task.  # noqa: E501
        :rtype: ErrorReport
        """
        return self._error_report

    @error_report.setter
    def error_report(self, error_report):
        """Sets the error_report of this Task.


        :param error_report: The error_report of this Task.  # noqa: E501
        :type: ErrorReport
        """

        self._error_report = error_report

    @property
    def http_status(self):
        """Gets the http_status of this Task.  # noqa: E501


        :return: The http_status of this Task.  # noqa: E501
        :rtype: int
        """
        return self._http_status

    @http_status.setter
    def http_status(self, http_status):
        """Sets the http_status of this Task.


        :param http_status: The http_status of this Task.  # noqa: E501
        :type: int
        """

        self._http_status = http_status

    @property
    def duration(self):
        """Gets the duration of this Task.  # noqa: E501


        :return: The duration of this Task.  # noqa: E501
        :rtype: int
        """
        return self._duration

    @duration.setter
    def duration(self, duration):
        """Sets the duration of this Task.


        :param duration: The duration of this Task.  # noqa: E501
        :type: int
        """

        self._duration = duration

    @property
    def type(self):
        """Gets the type of this Task.  # noqa: E501


        :return: The type of this Task.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this Task.


        :param type: The type of this Task.  # noqa: E501
        :type: str
        """
        allowed_values = ["TRAINING", "PREDICTION", "PREPARATION", "VALIDATION"]  # noqa: E501
        if type not in allowed_values:
            raise ValueError(
                "Invalid value for `type` ({0}), must be one of {1}"  # noqa: E501
                .format(type, allowed_values)
            )

        self._type = type

    @property
    def id(self):
        """Gets the id of this Task.  # noqa: E501


        :return: The id of this Task.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Task.


        :param id: The id of this Task.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def status(self):
        """Gets the status of this Task.  # noqa: E501


        :return: The status of this Task.  # noqa: E501
        :rtype: str
        """
        return self._status

    @status.setter
    def status(self, status):
        """Sets the status of this Task.


        :param status: The status of this Task.  # noqa: E501
        :type: str
        """
        allowed_values = ["RUNNING", "COMPLETED", "CANCELLED", "ERROR", "REJECTED", "QUEUED"]  # noqa: E501
        if status not in allowed_values:
            raise ValueError(
                "Invalid value for `status` ({0}), must be one of {1}"  # noqa: E501
                .format(status, allowed_values)
            )

        self._status = status



# from tornado import gen, httpclient
# from tornado.httputil import HTTPHeaders
from jaqpotpy.mappers import decode
import requests

feat_path = "feature"


def create_feature_sync(baseurl, api_key, json_feat):
    uri = baseurl + feat_path
    token = "Bearer " + api_key
    h = {"Content-type": "application/json",
         "Accept": "application/json",
         'Authorization': token}
    try:
        r = requests.post(uri, data=json_feat, headers=h, verify=False)
        return r.json()
    except Exception as e:
        print("Error 1: " + str(e))


# @gen.coroutine
# def create_feature_async(baseurl, api_key, json_feat):
#     uri = baseurl + feat_path
#     jclient = httpclient.AsyncHTTPClient()
#     h = HTTPHeaders({'Content-Type': 'application/json'})
#     h.add('Accept', 'application/json')
#     h.add('Authorization', "Bearer " + api_key)
#     try:
#         response = yield jclient.fetch(uri,
#                                        method='POST',
#                                        headers=h,
#                                        body=json_feat,
#                                        validate_cert=False)
#         resp = decode.decode_feature(response.body)
#         raise gen.Return(resp)
#     except httpclient.HTTPError as e:
#         print("Error 1: " + str(e))
#     finally:
#         jclient.close()


# async def create_feature_async_(baseurl, api_key, json_feat):
#     uri = baseurl + feat_path
#     jclient = httpclient.AsyncHTTPClient()
#     h = HTTPHeaders({'Content-Type': 'application/json'})
#     h.add('Accept', 'application/json')
#     h.add('Authorization', "Bearer " + api_key)
#     try:
#         response = await jclient.fetch(uri, method='POST', headers=h, body=json_feat, validate_cert=False)
#         # au_req = decode.decode_feature(response.body)
#         # print(response.body)
#         # print(gen.Return(decode.decode_feature(response.body)))
#     except httpclient.HTTPError as e:
#         print("Error: " + str(e))
#     except Exception as e:
#         print("Error: " + str(e))
#     else:
#         print(decode.decode_feature(response.body))
#     jclient.close()
#
#
# def create_feature_sync(baseurl, api_key, json_feat):
#     uri = baseurl + feat_path
#     jclient = httpclient.HTTPClient()
#     h = HTTPHeaders({'Content-Type': 'application/json'})
#     h.add('Accept', 'application/json')
#     h.add('Authorization', "Bearer " + api_key)
#     try:
#         response = jclient.fetch(uri, method='POST', headers=h, body=json_feat, validate_cert=False)
#         au_req = decode.decode_feature(response.body)
#         return au_req
#     except httpclient.HTTPError as e:
#         print("Error: " + str(e))
#     except Exception as e:
#         print("Error: " + str(e))
#     jclient.close()



from .meta import MetaInfo  # noqa: F401,E501


class Feature(object):

    def __init__(self, meta=None, ontological_classes=None, visible=False, temporary=False, featured=False, units=None, predictor_for=None, admissible_values=None, actual_independent_feature_name=None, from_pretrained=False, id=None):

        self.meta = None
        self.ontologicalClasses = None
        self.visible = None
        self.temporary = None
        self.featured = None
        self.units = None
        self.predictorFor = None
        self.admissibleValues = None
        self.actualIndependentFeatureName = None
        self.fromPretrained = None
        self.id = None
        self.discriminator = None


        # if meta is not None:
        #     self.meta = meta
        # if ontological_classes is not None:
        #     self.ontological_classes = ontological_classes
        # if visible is not None:
        #     self.visible = visible
        # if temporary is not None:
        #     self.temporary = temporary
        # if featured is not None:
        #     self.featured = featured
        # if units is not None:
        #     self.units = units
        # if predictor_for is not None:
        #     self.predictor_for = predictor_for
        # if admissible_values is not None:
        #     self.admissible_values = admissible_values
        # if actual_independent_feature_name is not None:
        #     self.actual_independent_feature_name = actual_independent_feature_name
        # if from_pretrained is not None:
        #     self.from_pretrained = from_pretrained
        # if id is not None:
        #     self.id = id

    # @property
    # def meta(self):
    #     """Gets the meta of this Feature.  # noqa: E501
    #
    #
    #     :return: The meta of this Feature.  # noqa: E501
    #     :rtype: MetaInfo
    #     """
    #     return self._meta
    #
    # @meta.setter
    # def meta(self, meta):
    #     """Sets the meta of this Feature.
    #
    #
    #     :param meta: The meta of this Feature.  # noqa: E501
    #     :type: MetaInfo
    #     """
    #
    #     self._meta = meta
    #
    # @property
    # def ontological_classes(self):
    #     """Gets the ontological_classes of this Feature.  # noqa: E501
    #
    #
    #     :return: The ontological_classes of this Feature.  # noqa: E501
    #     :rtype: list[str]
    #     """
    #     return self._ontological_classes
    #
    # @ontological_classes.setter
    # def ontological_classes(self, ontological_classes):
    #     """Sets the ontological_classes of this Feature.
    #
    #
    #     :param ontological_classes: The ontological_classes of this Feature.  # noqa: E501
    #     :type: list[str]
    #     """
    #
    #     self._ontological_classes = ontological_classes
    #
    # @property
    # def visible(self):
    #     """Gets the visible of this Feature.  # noqa: E501
    #
    #
    #     :return: The visible of this Feature.  # noqa: E501
    #     :rtype: bool
    #     """
    #     return self._visible
    #
    # @visible.setter
    # def visible(self, visible):
    #     """Sets the visible of this Feature.
    #
    #
    #     :param visible: The visible of this Feature.  # noqa: E501
    #     :type: bool
    #     """
    #
    #     self._visible = visible
    #
    # @property
    # def temporary(self):
    #     """Gets the temporary of this Feature.  # noqa: E501
    #
    #
    #     :return: The temporary of this Feature.  # noqa: E501
    #     :rtype: bool
    #     """
    #     return self._temporary
    #
    # @temporary.setter
    # def temporary(self, temporary):
    #     """Sets the temporary of this Feature.
    #
    #
    #     :param temporary: The temporary of this Feature.  # noqa: E501
    #     :type: bool
    #     """
    #
    #     self._temporary = temporary
    #
    # @property
    # def featured(self):
    #     """Gets the featured of this Feature.  # noqa: E501
    #
    #
    #     :return: The featured of this Feature.  # noqa: E501
    #     :rtype: bool
    #     """
    #     return self._featured
    #
    # @featured.setter
    # def featured(self, featured):
    #     """Sets the featured of this Feature.
    #
    #
    #     :param featured: The featured of this Feature.  # noqa: E501
    #     :type: bool
    #     """
    #
    #     self._featured = featured
    #
    # @property
    # def units(self):
    #     """Gets the units of this Feature.  # noqa: E501
    #
    #
    #     :return: The units of this Feature.  # noqa: E501
    #     :rtype: str
    #     """
    #     return self._units
    #
    # @units.setter
    # def units(self, units):
    #     """Sets the units of this Feature.
    #
    #
    #     :param units: The units of this Feature.  # noqa: E501
    #     :type: str
    #     """
    #
    #     self._units = units
    #
    # @property
    # def predictor_for(self):
    #     """Gets the predictor_for of this Feature.  # noqa: E501
    #
    #
    #     :return: The predictor_for of this Feature.  # noqa: E501
    #     :rtype: str
    #     """
    #     return self._predictor_for
    #
    # @predictor_for.setter
    # def predictor_for(self, predictor_for):
    #     """Sets the predictor_for of this Feature.
    #
    #
    #     :param predictor_for: The predictor_for of this Feature.  # noqa: E501
    #     :type: str
    #     """
    #
    #     self._predictor_for = predictor_for
    #
    # @property
    # def admissible_values(self):
    #     """Gets the admissible_values of this Feature.  # noqa: E501
    #
    #
    #     :return: The admissible_values of this Feature.  # noqa: E501
    #     :rtype: list[str]
    #     """
    #     return self._admissible_values
    #
    # @admissible_values.setter
    # def admissible_values(self, admissible_values):
    #     """Sets the admissible_values of this Feature.
    #
    #
    #     :param admissible_values: The admissible_values of this Feature.  # noqa: E501
    #     :type: list[str]
    #     """
    #
    #     self._admissible_values = admissible_values
    #
    # @property
    # def actual_independent_feature_name(self):
    #     """Gets the actual_independent_feature_name of this Feature.  # noqa: E501
    #
    #
    #     :return: The actual_independent_feature_name of this Feature.  # noqa: E501
    #     :rtype: str
    #     """
    #     return self._actual_independent_feature_name
    #
    # @actual_independent_feature_name.setter
    # def actual_independent_feature_name(self, actual_independent_feature_name):
    #     """Sets the actual_independent_feature_name of this Feature.
    #
    #
    #     :param actual_independent_feature_name: The actual_independent_feature_name of this Feature.  # noqa: E501
    #     :type: str
    #     """
    #
    #     self._actual_independent_feature_name = actual_independent_feature_name
    #
    # @property
    # def from_pretrained(self):
    #     """Gets the from_pretrained of this Feature.  # noqa: E501
    #
    #
    #     :return: The from_pretrained of this Feature.  # noqa: E501
    #     :rtype: bool
    #     """
    #     return self._from_pretrained
    #
    # @from_pretrained.setter
    # def from_pretrained(self, from_pretrained):
    #     """Sets the from_pretrained of this Feature.
    #
    #
    #     :param from_pretrained: The from_pretrained of this Feature.  # noqa: E501
    #     :type: bool
    #     """
    #
    #     self._from_pretrained = from_pretrained
    #
    # @property
    # def id(self):
    #     """Gets the id of this Feature.  # noqa: E501
    #
    #
    #     :return: The id of this Feature.  # noqa: E501
    #     :rtype: str
    #     """
    #     return self._id
    #
    # @id.setter
    # def id(self, id):
    #     """Sets the id of this Feature.
    #
    #
    #     :param id: The id of this Feature.  # noqa: E501
    #     :type: str
    #     """
    #
    #     self._id = id



class PretrainedRequirements:

    def __init__(self, rawModel=None, independentFeatures=None,
                 predictedFeatures=None, additionalInfo=None,
                 title=None, implementedWith=None,
                 algorithm=None, implementedIn=None,
                 description=None, dependentFeatures=None):
        self.rawModel = rawModel
        self.independentFeatures = independentFeatures
        self.predictedFeatures = predictedFeatures
        self.dependentFeatures = dependentFeatures
        self.additionalInfo = additionalInfo
        self.title = title
        self.implementedWith = implementedWith
        self.implementedIn = implementedIn
        self.algorithm = algorithm
        self.description = description

        # if rawModel is not None:
        #     self.rawModel = rawModel
        # if independentFeatures is not None:
        #     self.independentFeatures = independentFeatures
        # if predictedFeatures is not None:
        #     self.predictedFeatures = predictedFeatures
        # if title is not None:
        #     self.title = title
        # if implementedWith is not None:
        #     self.implementedWith = implementedWith
        # if algorithm is not None:
        #     self.algorithm = algorithm
        # if implementedIn is not None:
        #     self.implementedIn = implementedIn
        # if description is not None:
        #     self.description = description
        # if dependentFeatures is not None:
        #     self.dependentFeatures = dependentFeatures



from .meta import MetaInfo
from .parameter import Parameter


class Algorithm(object):

    def __init__(self, meta=None, ontological_classes=None, visible=False, temporary=False,
                 featured=False, parameters=None, ranking=None, bibtex=None, training_service=None,
                 prediction_service=None, report_service=None, _id=None):
        self._meta = None
        self._ontological_classes = None
        self._visible = None
        self._temporary = None
        self._featured = None
        self._parameters = None
        self._ranking = None
        self._bibtex = None
        self._training_service = None
        self._prediction_service = None
        self._report_service = None
        self._id = None
        self.discriminator = None

        if meta is not None:
            self.meta = meta
        if ontological_classes is not None:
            self.ontological_classes = ontological_classes
        if visible is not None:
            self.visible = visible
        if temporary is not None:
            self.temporary = temporary
        if featured is not None:
            self.featured = featured
        if parameters is not None:
            self.parameters = parameters
        if ranking is not None:
            self.ranking = ranking
        if bibtex is not None:
            self.bibtex = bibtex
        if training_service is not None:
            self.training_service = training_service
        if prediction_service is not None:
            self.prediction_service = prediction_service
        if report_service is not None:
            self.report_service = report_service
        if _id is not None:
            self._id = _id

    # @property
    # def meta(self):
    #     """Gets the meta of this Algorithm.  # noqa: E501
    #
    #
    #     :return: The meta of this Algorithm.  # noqa: E501
    #     :rtype: MetaInfo
    #     """
    #     return self._meta
    #
    # @meta.setter
    # def meta(self, meta):
    #     """Sets the meta of this Algorithm.
    #
    #
    #     :param meta: The meta of this Algorithm.  # noqa: E501
    #     :type: MetaInfo
    #     """
    #
    #     self._meta = meta
    #
    # @property
    # def ontological_classes(self):
    #     """Gets the ontological_classes of this Algorithm.  # noqa: E501
    #
    #
    #     :return: The ontological_classes of this Algorithm.  # noqa: E501
    #     :rtype: list[str]
    #     """
    #     return self._ontological_classes
    #
    # @ontological_classes.setter
    # def ontological_classes(self, ontological_classes):
    #     """Sets the ontological_classes of this Algorithm.
    #
    #
    #     :param ontological_classes: The ontological_classes of this Algorithm.  # noqa: E501
    #     :type: list[str]
    #     """
    #
    #     self._ontological_classes = ontological_classes
    #
    # @property
    # def visible(self):
    #     """Gets the visible of this Algorithm.  # noqa: E501
    #
    #
    #     :return: The visible of this Algorithm.  # noqa: E501
    #     :rtype: bool
    #     """
    #     return self._visible
    #
    # @visible.setter
    # def visible(self, visible):
    #     """Sets the visible of this Algorithm.
    #
    #
    #     :param visible: The visible of this Algorithm.  # noqa: E501
    #     :type: bool
    #     """
    #
    #     self._visible = visible
    #
    # @property
    # def temporary(self):
    #     """Gets the temporary of this Algorithm.  # noqa: E501
    #
    #
    #     :return: The temporary of this Algorithm.  # noqa: E501
    #     :rtype: bool
    #     """
    #     return self._temporary
    #
    # @temporary.setter
    # def temporary(self, temporary):
    #     """Sets the temporary of this Algorithm.
    #
    #
    #     :param temporary: The temporary of this Algorithm.  # noqa: E501
    #     :type: bool
    #     """
    #
    #     self._temporary = temporary
    #
    # @property
    # def featured(self):
    #     """Gets the featured of this Algorithm.  # noqa: E501
    #
    #
    #     :return: The featured of this Algorithm.  # noqa: E501
    #     :rtype: bool
    #     """
    #     return self._featured
    #
    # @featured.setter
    # def featured(self, featured):
    #     """Sets the featured of this Algorithm.
    #
    #
    #     :param featured: The featured of this Algorithm.  # noqa: E501
    #     :type: bool
    #     """
    #
    #     self._featured = featured
    #
    # @property
    # def parameters(self):
    #     """Gets the parameters of this Algorithm.  # noqa: E501
    #
    #
    #     :return: The parameters of this Algorithm.  # noqa: E501
    #     :rtype: list[Parameter]
    #     """
    #     return self._parameters
    #
    # @parameters.setter
    # def parameters(self, parameters):
    #     """Sets the parameters of this Algorithm.
    #
    #
    #     :param parameters: The parameters of this Algorithm.  # noqa: E501
    #     :type: list[Parameter]
    #     """
    #
    #     self._parameters = parameters
    #
    # @property
    # def ranking(self):
    #     """Gets the ranking of this Algorithm.  # noqa: E501
    #
    #
    #     :return: The ranking of this Algorithm.  # noqa: E501
    #     :rtype: int
    #     """
    #     return self._ranking
    #
    # @ranking.setter
    # def ranking(self, ranking):
    #     """Sets the ranking of this Algorithm.
    #
    #
    #     :param ranking: The ranking of this Algorithm.  # noqa: E501
    #     :type: int
    #     """
    #
    #     self._ranking = ranking
    #
    # @property
    # def bibtex(self):
    #     """Gets the bibtex of this Algorithm.  # noqa: E501
    #
    #
    #     :return: The bibtex of this Algorithm.  # noqa: E501
    #     :rtype: list[BibTeX]
    #     """
    #     return self._bibtex
    #
    # @bibtex.setter
    # def bibtex(self, bibtex):
    #     """Sets the bibtex of this Algorithm.
    #
    #
    #     :param bibtex: The bibtex of this Algorithm.  # noqa: E501
    #     :type: list[BibTeX]
    #     """
    #
    #     self._bibtex = bibtex
    #
    # @property
    # def training_service(self):
    #     """Gets the training_service of this Algorithm.  # noqa: E501
    #
    #
    #     :return: The training_service of this Algorithm.  # noqa: E501
    #     :rtype: str
    #     """
    #     return self._training_service
    #
    # @training_service.setter
    # def training_service(self, training_service):
    #     """Sets the training_service of this Algorithm.
    #
    #
    #     :param training_service: The training_service of this Algorithm.  # noqa: E501
    #     :type: str
    #     """
    #
    #     self._training_service = training_service
    #
    # @property
    # def prediction_service(self):
    #     """Gets the prediction_service of this Algorithm.  # noqa: E501
    #
    #
    #     :return: The prediction_service of this Algorithm.  # noqa: E501
    #     :rtype: str
    #     """
    #     return self._prediction_service
    #
    # @prediction_service.setter
    # def prediction_service(self, prediction_service):
    #     """Sets the prediction_service of this Algorithm.
    #
    #
    #     :param prediction_service: The prediction_service of this Algorithm.  # noqa: E501
    #     :type: str
    #     """
    #
    #     self._prediction_service = prediction_service
    #
    # @property
    # def report_service(self):
    #     """Gets the report_service of this Algorithm.  # noqa: E501
    #
    #
    #     :return: The report_service of this Algorithm.  # noqa: E501
    #     :rtype: str
    #     """
    #     return self._report_service
    #
    # @report_service.setter
    # def report_service(self, report_service):
    #     """Sets the report_service of this Algorithm.
    #
    #
    #     :param report_service: The report_service of this Algorithm.  # noqa: E501
    #     :type: str
    #     """
    #
    #     self._report_service = report_service
    #
    # @property
    # def _id(self):
    #     """Gets the id of this Algorithm.  # noqa: E501
    #
    #
    #     :return: The id of this Algorithm.  # noqa: E501
    #     :rtype: str
    #     """
    #     return self._id
    #
    # @_id.setter
    # def _id(self, _id):
    #     """Sets the id of this Algorithm.
    #
    #
    #     :param _id: The id of this Algorithm.  # noqa: E501
    #     :type: str
    #     """
    #
    #     self._id = _id



from tornado import gen, httpclient
from tornado.httputil import HTTPHeaders
import getpass
import urllib.parse
from jaqpotpy.mappers import decode
import http.client
import requests

login_path = "aa/login"


def authenticate_sync(baseurl, username, password):
    uri = baseurl + login_path
    data = {
        'username': username,
        'password': password
    }
    body = urllib.parse.urlencode(data)
    h = {"Content-type": "application/x-www-form-urlencoded",
         "Accept": "application/json"}
    try:
        r = requests.post(uri, data=body, headers=h, verify=False)
        # resp = decode.decode_auth(r.text)
        return r.json()
    except Exception as e:
        print("Error 1: " + str(e))


# def authenticate_sync(http_client, baseurl, username, password):
#     uri = baseurl + login_path
#     jclient = httpclient.HTTPClient()
#     h = HTTPHeaders({'Content-Type': 'application/x-www-form-urlencoded'})
#     h.add('Accept', 'application/json')
#     data = {
#         'username': username,
#         'password': password
#     }
#     body = urllib.parse.urlencode(data)
#     try:
#         response = jclient.fetch(uri, method='POST', headers=h, body=body, validate_cert=False)
#         au_req = decode.decode_auth(response.body)
#         return au_req
#     except httpclient.HTTPError as e:
#         print("Error: " + str(e))
#     except Exception as e:
#         print("Error: " + str(e))
#     jclient.close()


# @gen.coroutine
# def authenticate_async(baseurl, username, password):
#     uri = baseurl + login_path
#     jclient = httpclient.AsyncHTTPClient()
#     h = HTTPHeaders({'Content-Type': 'application/x-www-form-urlencoded'})
#     h.add('Accept', 'application/json')
#     data = {
#         'username': username,
#         'password': password
#     }
#     body = urllib.parse.urlencode(data)
#     try:
#         response = yield jclient.fetch(uri, method='POST', headers=h, body=body, validate_cert=False)
#         au_req = decode.decode_auth(response.body)
#         raise gen.Return(au_req)
#     except httpclient.HTTPError as e:
#         print("Error: " + str(e))
#     except Exception as e:
#         print("Error: " + str(e))
#     jclient.close()
#
#
# @gen.coroutine
# def authenticate_async_hidepass(baseurl):
#     uri = baseurl + login_path
#     jclient = httpclient.AsyncHTTPClient()
#     username = input("Username: ")
#     password = getpass.getpass("Password: ")
#     h = HTTPHeaders({'Content-Type': 'application/x-www-form-urlencoded'})
#     h.add('Accept', 'application/json')
#     data = {
#         'username': username,
#         'password': password
#     }
#     body = urllib.parse.urlencode(data)
#     try:
#         response = yield jclient.fetch(uri, method='POST', headers=h, body=body, validate_cert=False)
#         au_req = decode.decode_auth(response.body)
#         raise gen.Return(au_req)
#     except httpclient.HTTPError as e:
#         print("Error: " + str(e))
#     except Exception as e:
#         print("Error: " + str(e))
#     jclient.close()
#
#
# def authenticate_sync_hidepass(baseurl):
#     uri = baseurl + login_path
#     jclient = httpclient.HTTPClient()
#     username = input("Username: ")
#     password = getpass.getpass("Password: ")
#     h = HTTPHeaders({'Content-Type': 'application/x-www-form-urlencoded'})
#     h.add('Accept', 'application/json')
#     data = {
#         'username': username,
#         'password': password
#     }
#     body = urllib.parse.urlencode(data)
#     try:
#         response = jclient.fetch(uri, method='POST', headers=h, body=body, validate_cert=False)
#         au_req = decode.decode_auth(response.body)
#         return au_req
#     except httpclient.HTTPError as e:
#         print("Error: " + str(e))
#     except Exception as e:
#         print("Error: " + str(e))
#     jclient.close()



from jaqpotpy.entities.feature import Feature
import math
from jaqpotpy.helpers.builders import FeatureBuilder,\
    FeatureDirector, DataEntryBuilder, DataEntryDirector,\
    PretrainedNeedsDirector, PretrainedNeedsBuilder


def create_feature(feat_title, creator):
    fb = FeatureBuilder()
    fb.add_comments("Feature created from python client for dataset creation")
    fb.add_creator(creator)
    fb.add_title(feat_title)
    dire = FeatureDirector()
    mf = dire.construct(fb)
    return mf


def clear_entity(o):
    delete = []
    for key in o:
        if getattr(o, key) is None or "":
            delete.append(key)
    for keytod in delete:
        delattr(o, keytod)
    return o


def create_data_entry(df, feat_map, owner_uuid):
    data_entry = []
    for dataid in df.index:
        # data_entry_all = {}
        values_from_dataframe = df.loc[dataid]

        if type(values_from_dataframe).__name__ is not 'DataFrame':
            values_from_dataframe = values_from_dataframe.to_frame()
        # values_from_dataframe = values_from_dataframe.to_json()
        # print(values_from_dataframe_j)
        # print(dataid)
        # print(values_from_dataframe)

        de_director = DataEntryDirector()
        de_builder = DataEntryBuilder()
        de_builder.set_name(dataid)
        de_builder.set_owneruuid(owner_uuid)
        values = {}
        for key in feat_map:
            values[feat_map[key]] = values_from_dataframe[dataid][key]
        de_builder.set_values(values)
        de = de_director.construct(de_builder)
        data_entry.append(de.__dict__)
    return data_entry


def create_pretrain_req(model, X, y, title, description, algorithm, implementedWith, runtime, additionalInfo):
    pnb = PretrainedNeedsBuilder()
    independentFeatures = []
    dependendFeatures = []
    for fe in list(X):
        independentFeatures.append(fe)
    if type(y).__name__ is 'Dataframe':
        for fe in list(y):
            dependendFeatures.append(fe)
    elif type(y).__name__ is 'Series':
        dependendFeatures.append(y.name)
    pnb.setRawModel(model)
    pnb.setAdditionalInfo(additionalInfo)
    pnb.setAlgorithm(algorithm)
    pnb.setDependendFeatures(dependendFeatures)
    pnb.setIndependentFeatures(independentFeatures)
    pnb.setDescription(description)
    pnb.setTitle(title)
    pnb.setRuntime(runtime)
    pnb.setImplementedWith(implementedWith)
    director = PretrainedNeedsDirector()
    return director.construct(pnb)



class Parameter(object):

    def __init__(self, required=False, description=None, vendor_extensions=None, pattern=None, _in=None, name=None):

        self._required = None
        self._description = None
        self._vendor_extensions = None
        self._pattern = None
        self.__in = None
        self._name = None
        self.discriminator = None

        if required is not None:
            self.required = required
        if description is not None:
            self.description = description
        if vendor_extensions is not None:
            self.vendor_extensions = vendor_extensions
        if pattern is not None:
            self.pattern = pattern
        if _in is not None:
            self._in = _in
        if name is not None:
            self.name = name

    @property
    def required(self):
        """Gets the required of this Parameter.  # noqa: E501


        :return: The required of this Parameter.  # noqa: E501
        :rtype: bool
        """
        return self._required

    @required.setter
    def required(self, required):
        """Sets the required of this Parameter.


        :param required: The required of this Parameter.  # noqa: E501
        :type: bool
        """

        self._required = required

    @property
    def description(self):
        """Gets the description of this Parameter.  # noqa: E501


        :return: The description of this Parameter.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this Parameter.


        :param description: The description of this Parameter.  # noqa: E501
        :type: str
        """

        self._description = description

    @property
    def vendor_extensions(self):
        """Gets the vendor_extensions of this Parameter.  # noqa: E501


        :return: The vendor_extensions of this Parameter.  # noqa: E501
        :rtype: dict(str, object)
        """
        return self._vendor_extensions

    @vendor_extensions.setter
    def vendor_extensions(self, vendor_extensions):
        """Sets the vendor_extensions of this Parameter.


        :param vendor_extensions: The vendor_extensions of this Parameter.  # noqa: E501
        :type: dict(str, object)
        """

        self._vendor_extensions = vendor_extensions

    @property
    def pattern(self):
        """Gets the pattern of this Parameter.  # noqa: E501


        :return: The pattern of this Parameter.  # noqa: E501
        :rtype: str
        """
        return self._pattern

    @pattern.setter
    def pattern(self, pattern):
        """Sets the pattern of this Parameter.


        :param pattern: The pattern of this Parameter.  # noqa: E501
        :type: str
        """

        self._pattern = pattern

    @property
    def _in(self):
        """Gets the _in of this Parameter.  # noqa: E501


        :return: The _in of this Parameter.  # noqa: E501
        :rtype: str
        """
        return self.__in

    @_in.setter
    def _in(self, _in):
        """Sets the _in of this Parameter.


        :param _in: The _in of this Parameter.  # noqa: E501
        :type: str
        """

        self.__in = _in

    @property
    def name(self):
        """Gets the name of this Parameter.  # noqa: E501


        :return: The name of this Parameter.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Parameter.


        :param name: The name of this Parameter.  # noqa: E501
        :type: str
        """

        self._name = name



from setuptools import setup

setup(name='jaqpotpy',
      version='0.0.1',
      description='Python client for Jaqpot',
      url='https://github.com/KinkyDesign/jaqpotpy',
      author='Pantelis Karatzas, Angleos Valsamis, Pantelis Sopasakis',
      author_email='pantelispanka@gmail.com',
      license='GNU General Public License v3.0',
      packages=['jaqpotpy', 'jaqpotpy.api', 'jaqpotpy.mappers', 'jaqpotpy.entities', 'jaqpotpy.dto', 'jaqpotpy.helpers'],
      install_requires=[
            'tornado==4.5.3', 'pandas', 'pyjwt', 'requests'
      ],
      zip_safe=False)



class AuthRequest:

    def __init__(self, username='', authtoken=''):
        self.userName = username
        self.authToken = authtoken

    # @property
    # def userName(self):
    #     return self._userName
    #
    # @property
    # def authToken(self):
    #     return self._authToken
    #
    # @userName.setter
    # def userName(self, value):
    #     self.userName = value
    #
    # @authToken.setter
    # def authToken(self, value):
    #     self.authToken = value
    #
    # @userName.deleter
    # def userName(self):
    #     del self.userName
    #
    # @authToken.deleter
    # def authToken(self):
    #     del self.authToken
